mod decode;
mod encode;

pub use {
    decode::{Decoder as B64Decoder, Error as B64DecoderError},
    encode::{Encoder as B64Encoder, Error as B64EncoderError},
};

#[derive(Clone, Copy)]
pub struct B64Alphabet {
    items: [char; 64],
    pad: char,
}

pub static B64_RFC4648_ALPHABET: B64Alphabet = B64Alphabet {
    items: [
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R',
        'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
        'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1',
        '2', '3', '4', '5', '6', '7', '8', '9', '+', '/',
    ],
    pad: '=',
};

impl Default for B64Alphabet {
    fn default() -> Self {
        B64_RFC4648_ALPHABET
    }
}

impl B64Alphabet {
    pub fn idx(&self, c: char) -> Option<usize> {
        for (idx, x) in self.items.iter().enumerate() {
            if *x == c {
                return Some(idx);
            }
        }
        None
    }

    pub fn pad(&self) -> char {
        self.pad
    }
}
